@@ -57,9 +57,6 @@ Street, Fifth Floor, Boston, MA 02110-1335 USA
 #include "backup_copy.h"
 #include "backup_mysql.h"
 #include <btr0btr.h>
-#ifdef _WIN32
-#include <direct.h> /* rmdir */
-#endif
 
 #define ROCKSDB_BACKUP_DIR "#rocksdb"
 
@@ -73,8 +70,9 @@ bool binlog_locked;
 
 static void rocksdb_create_checkpoint();
 static bool has_rocksdb_plugin();
-static void rocksdb_backup_checkpoint(ds_ctxt *ds_data);
-static void rocksdb_copy_back(ds_ctxt *ds_data);
+static void copy_or_move_dir(const char *from, const char *to, bool copy, bool allow_hardlinks);
+static void rocksdb_backup_checkpoint();
+static void rocksdb_copy_back();
 
 static bool is_abs_path(const char *path)
 {
@@ -130,9 +128,7 @@ struct datadir_thread_ctxt_t {
 	bool			ret;
 };
 
-static bool backup_files_from_datadir(ds_ctxt_t *ds_data,
-                                      const char *dir_path,
-                                      const char *prefix);
+static bool backup_files_from_datadir(const char *dir_path);
 
 /************************************************************************
 Retirn true if character if file separator */
@@ -469,14 +465,13 @@ struct datafile_cur_t {
 	char		abs_path[FN_REFLEN];
 	MY_STAT		statinfo;
 	uint		thread_n;
-	byte*		orig_buf;
 	byte*		buf;
 	size_t		buf_size;
 	size_t		buf_read;
 	size_t		buf_offset;
 
 	explicit datafile_cur_t(const char* filename = NULL) :
-		file(), thread_n(0), orig_buf(NULL), buf(NULL), buf_size(0),
+		file(), thread_n(0), buf(NULL), buf_size(0),
 		buf_read(0), buf_offset(0)
 	{
 		memset(rel_path, 0, sizeof rel_path);
@@ -805,7 +800,7 @@ if passes the rules for partial backup.
 @return true if file backed up or skipped successfully. */
 static
 bool
-datafile_copy_backup(ds_ctxt *ds_data, const char *filepath, uint thread_n)
+datafile_copy_backup(const char *filepath, uint thread_n)
 {
 	const char *ext_list[] = {"frm", "isl", "MYD", "MYI", "MAD", "MAI",
 		"MRG", "TRG", "TRN", "ARM", "ARZ", "CSM", "CSV", "opt", "par",
@@ -826,7 +821,7 @@ datafile_copy_backup(ds_ctxt *ds_data, const char *filepath, uint thread_n)
 	}
 
 	if (filename_matches(filepath, ext_list)) {
-		return ds_data->copy_file(filepath, filepath, thread_n);
+		return copy_file(ds_data, filepath, filepath, thread_n);
 	}
 
 	return(true);
@@ -867,8 +862,7 @@ datafile_rsync_backup(const char *filepath, bool save_to_list, FILE *f)
 	return(true);
 }
 
-bool ds_ctxt_t::backup_file_print_buf(const char *filename,
-                                      const char *buf, int buf_len)
+bool backup_file_print_buf(const char *filename, const char *buf, int buf_len)
 {
 	ds_file_t	*dstfile	= NULL;
 	MY_STAT		 stat;			/* unused for now */
@@ -879,7 +873,7 @@ bool ds_ctxt_t::backup_file_print_buf(const char *filename,
 	stat.st_size = buf_len;
 	stat.st_mtime = my_time(0);
 
-	dstfile = ds_open(this, filename, &stat);
+	dstfile = ds_open(ds_data, filename, &stat);
 	if (dstfile == NULL) {
 		msg("error: Can't open the destination stream for %s",
 			filename);
@@ -918,9 +912,9 @@ error_close:
 	return true;
 };
 
+static
 bool
-ds_ctxt_t::backup_file_vprintf(const char *filename,
-                               const char *fmt, va_list ap)
+backup_file_vprintf(const char *filename, const char *fmt, va_list ap)
 {
 	char		*buf		= 0;
 	int		 buf_len;
@@ -931,7 +925,7 @@ ds_ctxt_t::backup_file_vprintf(const char *filename,
 }
 
 bool
-ds_ctxt_t::backup_file_printf(const char *filename, const char *fmt, ...)
+backup_file_printf(const char *filename, const char *fmt, ...)
 {
 	bool result;
 	va_list ap;
@@ -965,7 +959,7 @@ run_data_threads(datadir_iter_t *it, os_thread_func_t func, uint n)
 		data_threads[i].n_thread = i + 1;
 		data_threads[i].count = &count;
 		data_threads[i].count_mutex = &count_mutex;
-		os_thread_create(func, data_threads + i, &data_threads[i].id);
+		data_threads[i].id = os_thread_create(func, data_threads + i);
 	}
 
 	/* Wait for threads to exit */
@@ -1036,16 +1030,16 @@ static int fix_win_file_permissions(const char *file)
 	ea.grfInheritance = CONTAINER_INHERIT_ACE | OBJECT_INHERIT_ACE;
 	ea.Trustee.TrusteeType = TRUSTEE_IS_UNKNOWN;
 	ACL* pNewDACL = 0;
-	SetEntriesInAcl(1, &ea, pOldDACL, &pNewDACL);
-	if (pNewDACL)
+	DWORD err = SetEntriesInAcl(1, &ea, pOldDACL, &pNewDACL);
+	if (!err)
 	{
+		DBUG_ASSERT(pNewDACL);
 		SetSecurityInfo(hFile, SE_FILE_OBJECT, DACL_SECURITY_INFORMATION, NULL, NULL,
 			pNewDACL, NULL);
+		LocalFree((HLOCAL)pNewDACL);
 	}
 	if (pSD != NULL)
 		LocalFree((HLOCAL)pSD);
-	if (pNewDACL != NULL)
-		LocalFree((HLOCAL)pNewDACL);
 	CloseHandle(hFile);
 	return 0;
 }
@@ -1057,15 +1051,16 @@ static int fix_win_file_permissions(const char *file)
 Copy file for backup/restore.
 @return true in case of success. */
 bool
-ds_ctxt_t::copy_file(const char *src_file_path,
-                     const char *dst_file_path,
-                     uint thread_n)
+copy_file(ds_ctxt_t *datasink,
+	  const char *src_file_path,
+	  const char *dst_file_path,
+	  uint thread_n)
 {
 	char			 dst_name[FN_REFLEN];
 	ds_file_t		*dstfile = NULL;
 	datafile_cur_t		 cursor;
 	xb_fil_cur_result_t	 res;
-	DBUG_ASSERT(datasink->remove);
+	DBUG_ASSERT(datasink->datasink->remove);
 	const char	*dst_path =
 		(xtrabackup_copy_back || xtrabackup_move_back)?
 		dst_file_path : trim_dotslash(dst_file_path);
@@ -1076,7 +1071,7 @@ ds_ctxt_t::copy_file(const char *src_file_path,
 
 	strncpy(dst_name, cursor.rel_path, sizeof(dst_name));
 
-	dstfile = ds_open(this, dst_path, &cursor.statinfo);
+	dstfile = ds_open(datasink, dst_path, &cursor.statinfo);
 	if (dstfile == NULL) {
 		msg(thread_n,"error: "
 			"cannot open the destination stream for %s", dst_name);
@@ -1113,7 +1108,7 @@ ds_ctxt_t::copy_file(const char *src_file_path,
 error:
 	datafile_close(&cursor);
 	if (dstfile != NULL) {
-		datasink->remove(dstfile->path);
+		datasink->datasink->remove(dstfile->path);
 		ds_close(dstfile);
 	}
 
@@ -1127,10 +1122,12 @@ error_close:
 Try to move file by renaming it. If source and destination are on
 different devices fall back to copy and unlink.
 @return true in case of success. */
+static
 bool
-ds_ctxt_t::move_file(const char *src_file_path,
-                     const char *dst_file_path,
-                     const char *dst_dir, uint thread_n)
+move_file(ds_ctxt_t *datasink,
+	  const char *src_file_path,
+	  const char *dst_file_path,
+	  const char *dst_dir, uint thread_n)
 {
 	char errbuf[MYSYS_STRERROR_SIZE];
 	char dst_file_path_abs[FN_REFLEN];
@@ -1157,7 +1154,7 @@ ds_ctxt_t::move_file(const char *src_file_path,
 	if (my_rename(src_file_path, dst_file_path_abs, MYF(0)) != 0) {
 		if (my_errno == EXDEV) {
 			/* Fallback to copy/unlink */
-			if(!copy_file(src_file_path,
+			if(!copy_file(datasink, src_file_path,
 					dst_file_path, thread_n))
 					return false;
 			msg(thread_n,"Removing %s", src_file_path);
@@ -1241,13 +1238,13 @@ Copy or move file depending on current mode.
 @return true in case of success. */
 static
 bool
-copy_or_move_file(ds_ctxt *datasink0, const char *src_file_path,
+copy_or_move_file(const char *src_file_path,
 		  const char *dst_file_path,
 		  const char *dst_dir,
 		  uint thread_n,
 		 bool copy = xtrabackup_copy_back)
 {
-	ds_ctxt_t *datasink = datasink0; /* copy to datadir by default */
+	ds_ctxt_t *datasink = ds_data;		/* copy to datadir by default */
 	char filedir[FN_REFLEN];
 	size_t filedir_len;
 	bool ret;
@@ -1295,13 +1292,13 @@ copy_or_move_file(ds_ctxt *datasink0, const char *src_file_path,
 	}
 
 	ret = (copy ?
-		datasink->copy_file(src_file_path, dst_file_path, thread_n) :
-		datasink->move_file(src_file_path, dst_file_path,
+		copy_file(datasink, src_file_path, dst_file_path, thread_n) :
+		move_file(datasink, src_file_path, dst_file_path,
 			  dst_dir, thread_n));
 
 cleanup:
 
-	if (datasink != datasink0) {
+	if (datasink != ds_data) {
 		ds_destroy(datasink);
 	}
 
@@ -1313,7 +1310,7 @@ cleanup:
 
 static
 bool
-backup_files(ds_ctxt *ds_data, const char *from, bool prep_mode)
+backup_files(const char *from, bool prep_mode)
 {
 	char rsync_tmpfile_name[FN_REFLEN];
 	FILE *rsync_tmpfile = NULL;
@@ -1351,7 +1348,7 @@ backup_files(ds_ctxt *ds_data, const char *from, bool prep_mode)
 				ret = datafile_rsync_backup(node.filepath,
 					!prep_mode, rsync_tmpfile);
 			} else {
-				ret = datafile_copy_backup(ds_data, node.filepath, 1);
+				ret = datafile_copy_backup(node.filepath, 1);
 			}
 			if (!ret) {
 				msg("Failed to copy file %s", node.filepath);
@@ -1362,7 +1359,7 @@ backup_files(ds_ctxt *ds_data, const char *from, bool prep_mode)
 			char path[FN_REFLEN];
 			snprintf(path, sizeof(path),
 				 "%s/db.opt", node.filepath);
-			if (!(ret = ds_data->backup_file_printf(
+			if (!(ret = backup_file_printf(
 					trim_dotslash(path), "%s", ""))) {
 				msg("Failed to create file %s", path);
 				goto out;
@@ -1451,6 +1448,7 @@ out:
 	return(ret);
 }
 
+void backup_fix_ddl(CorruptedPages &);
 
 lsn_t get_current_lsn(MYSQL *connection)
 {
@@ -1475,8 +1473,7 @@ lsn_t get_current_lsn(MYSQL *connection)
 lsn_t server_lsn_after_lock;
 extern void backup_wait_for_lsn(lsn_t lsn);
 /** Start --backup */
-bool backup_start(ds_ctxt *ds_data, ds_ctxt *ds_meta,
-                  CorruptedPages &corrupted_pages)
+bool backup_start(CorruptedPages &corrupted_pages)
 {
 	if (!opt_no_lock) {
 		if (opt_safe_slave_backup) {
@@ -1485,7 +1482,7 @@ bool backup_start(ds_ctxt *ds_data, ds_ctxt *ds_meta,
 			}
 		}
 
-		if (!backup_files(ds_data, fil_path_to_mysql_datadir, true)) {
+		if (!backup_files(fil_path_to_mysql_datadir, true)) {
 			return(false);
 		}
 
@@ -1497,15 +1494,11 @@ bool backup_start(ds_ctxt *ds_data, ds_ctxt *ds_meta,
 		server_lsn_after_lock = get_current_lsn(mysql_connection);
 	}
 
-	if (!backup_files(ds_data, fil_path_to_mysql_datadir, false)) {
+	if (!backup_files(fil_path_to_mysql_datadir, false)) {
 		return(false);
 	}
 
-        if (!backup_files_from_datadir(ds_data, fil_path_to_mysql_datadir,
-	                               "aws-kms-key") ||
-            !backup_files_from_datadir(ds_data,
-                                       aria_log_dir_path,
-                                       "aria_log")) {
+	if (!backup_files_from_datadir(fil_path_to_mysql_datadir)) {
 		return false;
 	}
 
@@ -1515,7 +1508,7 @@ bool backup_start(ds_ctxt *ds_data, ds_ctxt *ds_meta,
 
 	msg("Waiting for log copy thread to read lsn %llu", (ulonglong)server_lsn_after_lock);
 	backup_wait_for_lsn(server_lsn_after_lock);
-	corrupted_pages.backup_fix_ddl(ds_data, ds_meta);
+	backup_fix_ddl(corrupted_pages);
 
 	// There is no need to stop slave thread before coping non-Innodb data when
 	// --no-lock option is used because --no-lock option requires that no DDL or
@@ -1531,7 +1524,7 @@ bool backup_start(ds_ctxt *ds_data, ds_ctxt *ds_meta,
 	if (opt_slave_info) {
 		lock_binlog_maybe(mysql_connection);
 
-		if (!write_slave_info(ds_data, mysql_connection)) {
+		if (!write_slave_info(mysql_connection)) {
 			return(false);
 		}
 	}
@@ -1543,15 +1536,17 @@ bool backup_start(ds_ctxt *ds_data, ds_ctxt *ds_meta,
 	avoid that is to have a single process, i.e. merge innobackupex and
 	xtrabackup. */
 	if (opt_galera_info) {
-		if (!write_galera_info(ds_data, mysql_connection)) {
+		if (!write_galera_info(mysql_connection)) {
 			return(false);
 		}
+                // copied from xtrabackup. what is it needed for here?
+		write_current_binlog_file(mysql_connection);
 	}
 
 	if (opt_binlog_info == BINLOG_INFO_ON) {
 
 		lock_binlog_maybe(mysql_connection);
-		write_binlog_info(ds_data, mysql_connection);
+		write_binlog_info(mysql_connection);
 	}
 
 	if (have_flush_engine_logs && !opt_no_lock) {
@@ -1588,20 +1583,20 @@ void backup_release()
 static const char *default_buffer_pool_file = "ib_buffer_pool";
 
 /** Finish after backup_start() and backup_release() */
-bool backup_finish(ds_ctxt *ds_data)
+bool backup_finish()
 {
 	/* Copy buffer pool dump or LRU dump */
 	if (!opt_rsync && opt_galera_info) {
 		if (buffer_pool_filename && file_exists(buffer_pool_filename)) {
-			ds_data->copy_file(buffer_pool_filename, default_buffer_pool_file, 0);
+			copy_file(ds_data, buffer_pool_filename, default_buffer_pool_file, 0);
 		}
 		if (file_exists("ib_lru_dump")) {
-			ds_data->copy_file("ib_lru_dump", "ib_lru_dump", 0);
+			copy_file(ds_data, "ib_lru_dump", "ib_lru_dump", 0);
 		}
 	}
 
 	if (has_rocksdb_plugin()) {
-		rocksdb_backup_checkpoint(ds_data);
+		rocksdb_backup_checkpoint();
 	}
 
 	msg("Backup created in directory '%s'", xtrabackup_target_dir);
@@ -1613,11 +1608,11 @@ bool backup_finish(ds_ctxt *ds_data)
 			mysql_slave_position);
 	}
 
-	if (!write_backup_config_file(ds_data)) {
+	if (!write_backup_config_file()) {
 		return(false);
 	}
 
-	if (!write_xtrabackup_info(ds_data, mysql_connection, XTRABACKUP_INFO,
+	if (!write_xtrabackup_info(mysql_connection, XTRABACKUP_INFO,
 				    opt_history != 0, true)) {
 		return(false);
 	}
@@ -1625,49 +1620,7 @@ bool backup_finish(ds_ctxt *ds_data)
 	return(true);
 }
 
-
-/*
-  Drop all empty database directories in the base backup
-  that do not exists in the icremental backup.
-
-  This effectively re-plays all DROP DATABASE statements happened
-  in between base backup and incremental backup creation time.
-
-  Note, only checking if base_dir/db/ is empty is not enough,
-  because inc_dir/db/db.opt might have been dropped for some reasons,
-  which may also result into empty base_dir/db/.
-
-  Only the fact that at the same time:
-  - base_dir/db/ exists
-  - inc_dir/db/ does not exist
-  means that DROP DATABASE happened.
-*/
-static void
-ibx_incremental_drop_databases(const char *base_dir,
-                               const char *inc_dir)
-{
-	datadir_node_t node;
-	datadir_node_init(&node);
-	datadir_iter_t *it = datadir_iter_new(base_dir);
-
-	while (datadir_iter_next(it, &node)) {
-		if (node.is_empty_dir) {
-			char path[FN_REFLEN];
-			snprintf(path, sizeof(path), "%s/%s",
-				 inc_dir, node.filepath_rel);
-			if (!directory_exists(path, false)) {
-				msg("Removing %s", node.filepath);
-				rmdir(node.filepath);
-			}
-		}
-
-	}
-	datadir_iter_free(it);
-	datadir_node_free(&node);
-}
-
-
-static bool
+bool
 ibx_copy_incremental_over_full()
 {
 	const char *ext_list[] = {"frm", "isl", "MYD", "MYI", "MAD", "MAI",
@@ -1684,7 +1637,6 @@ ibx_copy_incremental_over_full()
 	bool ret = true;
 	char path[FN_REFLEN];
 	int i;
-	ds_ctxt *ds_data= NULL;
 
         DBUG_ASSERT(!opt_galera_info);
 	datadir_node_init(&node);
@@ -1712,20 +1664,15 @@ ibx_copy_incremental_over_full()
 				unlink(node.filepath_rel);
 			}
 
-			if (!(ret = ds_data->copy_file(node.filepath,
-						       node.filepath_rel, 1))) {
+			if (!(ret = copy_file(ds_data, node.filepath,
+						node.filepath_rel, 1))) {
 				msg("Failed to copy file %s",
 					node.filepath);
 				goto cleanup;
 			}
 		}
 
-		if (!(ret = backup_files_from_datadir(ds_data,
-						      xtrabackup_incremental_dir,
-						      "aws-kms-key")) ||
-		    !(ret = backup_files_from_datadir(ds_data,
-						      xtrabackup_incremental_dir,
-						      "aria_log")))
+		if (!(ret = backup_files_from_datadir(xtrabackup_incremental_dir)))
 			goto cleanup;
 
 		/* copy supplementary files */
@@ -1740,7 +1687,7 @@ ibx_copy_incremental_over_full()
 				if (file_exists(sup_files[i])) {
 					unlink(sup_files[i]);
 				}
-				ds_data->copy_file(path, sup_files[i], 0);
+				copy_file(ds_data, path, sup_files[i], 0);
 			}
 		}
 
@@ -1754,10 +1701,8 @@ ibx_copy_incremental_over_full()
 			if (my_mkdir(ROCKSDB_BACKUP_DIR, 0777, MYF(0))) {
 				die("my_mkdir failed for " ROCKSDB_BACKUP_DIR);
 			}
-			ds_data->copy_or_move_dir(path, ROCKSDB_BACKUP_DIR, true, true);
+			copy_or_move_dir(path, ROCKSDB_BACKUP_DIR, true, true);
 		}
-		ibx_incremental_drop_databases(xtrabackup_target_dir,
-					       xtrabackup_incremental_dir);
 	}
 
 
@@ -1825,61 +1770,13 @@ apply_log_finish()
 	return(true);
 }
 
-class Copy_back_dst_dir
-{
-  std::string buf;
-
-public:
-  const char *make(const char *path)
-  {
-    if (!path || !path[0])
-      return mysql_data_home;
-    if (is_absolute_path(path))
-      return path;
-    return buf.assign(mysql_data_home).append(path).c_str();
-  }
-};
-
-
-static inline bool
-is_aria_log_dir_file(const datadir_node_t &node)
-{
-  return starts_with(node.filepath_rel, "aria_log");
-}
-
-
-bool
-copy_back_aria_logs(const char *dstdir)
-{
-  std::unique_ptr<ds_ctxt_t, void (&)(ds_ctxt_t*)>
-    ds_ctxt_aria_log_dir_path(ds_create(dstdir, DS_TYPE_LOCAL), ds_destroy);
-
-  datadir_node_t node;
-  datadir_node_init(&node);
-  datadir_iter_t *it = datadir_iter_new(".", false);
-
-  while (datadir_iter_next(it, &node))
-  {
-    if (!is_aria_log_dir_file(node))
-      continue;
-    if (!copy_or_move_file(ds_ctxt_aria_log_dir_path.get(),
-                           node.filepath, node.filepath_rel,
-                           dstdir, 1))
-      return false;
-  }
-  datadir_node_free(&node);
-  datadir_iter_free(it);
-  return true;
-}
-
-
 bool
 copy_back()
 {
 	bool ret = false;
 	datadir_iter_t *it = NULL;
 	datadir_node_t node;
-	const char *dst_dir;
+	char *dst_dir;
 
 	memset(&node, 0, sizeof(node));
 
@@ -1906,13 +1803,6 @@ copy_back()
 			return(false);
 	}
 
-        Copy_back_dst_dir aria_log_dir_path_dst;
-        const char *aria_log_dir_path_abs= aria_log_dir_path_dst.make(aria_log_dir_path);
-	if (aria_log_dir_path && *aria_log_dir_path
-		&& !directory_exists(aria_log_dir_path_abs, true)) {
-			return false;
-	}
-
 	/* cd to backup directory */
 	if (my_setwd(xtrabackup_target_dir, MYF(MY_WME)))
 	{
@@ -1920,9 +1810,6 @@ copy_back()
 		return(false);
 	}
 
-	if (!copy_back_aria_logs(aria_log_dir_path_abs))
-		return false;
-
 	/* parse data file path */
 
 	if (!innobase_data_file_path) {
@@ -1938,15 +1825,14 @@ copy_back()
 
 	srv_max_n_threads = 1000;
 	sync_check_init();
-	ut_crc32_init();
 
 	/* copy undo tablespaces */
 
-	Copy_back_dst_dir dst_dir_buf;
 
-	dst_dir = dst_dir_buf.make(srv_undo_dir);
+	dst_dir = (srv_undo_dir && *srv_undo_dir)
+			? srv_undo_dir : mysql_data_home;
 
-	ds_ctxt *ds_tmp = ds_create(dst_dir, DS_TYPE_LOCAL);
+	ds_data = ds_create(dst_dir, DS_TYPE_LOCAL);
 
 	for (uint i = 1; i <= TRX_SYS_MAX_UNDO_SPACES; i++) {
 		char filename[20];
@@ -1954,49 +1840,50 @@ copy_back()
 		if (!file_exists(filename)) {
 			break;
 		}
-		if (!(ret = copy_or_move_file(ds_tmp, filename, filename,
+		if (!(ret = copy_or_move_file(filename, filename,
 					      dst_dir, 1))) {
 			goto cleanup;
 		}
 	}
 
-	ds_destroy(ds_tmp);
-	ds_tmp = NULL;
+	ds_destroy(ds_data);
+	ds_data = NULL;
 
 	/* copy redo logs */
 
-	dst_dir = dst_dir_buf.make(srv_log_group_home_dir);
+	dst_dir = (srv_log_group_home_dir && *srv_log_group_home_dir)
+		? srv_log_group_home_dir : mysql_data_home;
 
-	/* --backup generates a single ib_logfile0, which we must copy
+	/* --backup generates a single LOG_FILE_NAME, which we must copy
 	if it exists. */
 
-	ds_tmp = ds_create(dst_dir, DS_TYPE_LOCAL);
+	ds_data = ds_create(dst_dir, DS_TYPE_LOCAL);
 	MY_STAT stat_arg;
-	if (!my_stat("ib_logfile0", &stat_arg, MYF(0)) || !stat_arg.st_size) {
+	if (!my_stat(LOG_FILE_NAME, &stat_arg, MYF(0)) || !stat_arg.st_size) {
 		/* After completed --prepare, redo log files are redundant.
 		We must delete any redo logs at the destination, so that
 		the database will not jump to a different log sequence number
 		(LSN). */
 
-		for (uint i = 0; i <= SRV_N_LOG_FILES_MAX + 1; i++) {
-			char filename[FN_REFLEN];
-			snprintf(filename, sizeof filename, "%s/ib_logfile%u",
-				 dst_dir, i);
-			unlink(filename);
-		}
-	}
-
-	if (!(ret = copy_or_move_file(ds_tmp, "ib_logfile0", "ib_logfile0",
-				      dst_dir, 1))) {
+		char filename[FN_REFLEN];
+		snprintf(filename, sizeof filename, "%s/%s0", dst_dir,
+			 LOG_FILE_NAME_PREFIX);
+		unlink(filename);
+		snprintf(filename, sizeof filename, "%s/%s101", dst_dir,
+			 LOG_FILE_NAME_PREFIX);
+		unlink(filename);
+	} else if (!(ret = copy_or_move_file(LOG_FILE_NAME, LOG_FILE_NAME,
+					     dst_dir, 1))) {
 		goto cleanup;
 	}
-	ds_destroy(ds_tmp);
+	ds_destroy(ds_data);
 
 	/* copy innodb system tablespace(s) */
 
-	dst_dir = dst_dir_buf.make(innobase_data_home_dir);
+	dst_dir = (innobase_data_home_dir && *innobase_data_home_dir)
+				? innobase_data_home_dir : mysql_data_home;
 
-	ds_tmp = ds_create(dst_dir, DS_TYPE_LOCAL);
+	ds_data = ds_create(dst_dir, DS_TYPE_LOCAL);
 
 	for (Tablespace::const_iterator iter(srv_sys_space.begin()),
 	     end(srv_sys_space.end());
@@ -2004,16 +1891,16 @@ copy_back()
 	     ++iter) {
 		const char *filename = base_name(iter->name());
 
-		if (!(ret = copy_or_move_file(ds_tmp, filename, iter->name(),
+		if (!(ret = copy_or_move_file(filename, iter->name(),
 					      dst_dir, 1))) {
 			goto cleanup;
 		}
 	}
 
-	ds_destroy(ds_tmp);
+	ds_destroy(ds_data);
 
 	/* copy the rest of tablespaces */
-	ds_tmp = ds_create(mysql_data_home, DS_TYPE_LOCAL);
+	ds_data = ds_create(mysql_data_home, DS_TYPE_LOCAL);
 
 	it = datadir_iter_new(".", false);
 
@@ -2029,10 +1916,6 @@ copy_back()
 		int i_tmp;
 		bool is_ibdata_file;
 
-		/* Skip aria log files */
-		if (is_aria_log_dir_file(node))
-			continue;
-
 		if (strstr(node.filepath,"/" ROCKSDB_BACKUP_DIR "/")
 #ifdef _WIN32
 			|| strstr(node.filepath,"\\" ROCKSDB_BACKUP_DIR "\\")
@@ -2081,7 +1964,7 @@ copy_back()
 		}
 
 		/* skip the redo log (it was already copied) */
-		if (!strcmp(filename, "ib_logfile0")) {
+		if (!strcmp(filename, LOG_FILE_NAME)) {
 			continue;
 		}
 
@@ -2104,7 +1987,7 @@ copy_back()
 			continue;
 		}
 
-		if (!(ret = copy_or_move_file(ds_tmp, node.filepath, node.filepath_rel,
+		if (!(ret = copy_or_move_file(node.filepath, node.filepath_rel,
 					      mysql_data_home, 1))) {
 			goto cleanup;
 		}
@@ -2114,12 +1997,12 @@ copy_back()
 
         if (file_exists(default_buffer_pool_file) &&
             innobase_buffer_pool_filename) {
-		copy_or_move_file(ds_tmp, default_buffer_pool_file,
+		copy_or_move_file(default_buffer_pool_file,
 				  innobase_buffer_pool_filename,
 				  mysql_data_home, 0);
 	}
 
-	rocksdb_copy_back(ds_tmp);
+	rocksdb_copy_back();
 
 cleanup:
 	if (it != NULL) {
@@ -2128,11 +2011,11 @@ cleanup:
 
 	datadir_node_free(&node);
 
-	if (ds_tmp != NULL) {
-		ds_destroy(ds_tmp);
+	if (ds_data != NULL) {
+		ds_destroy(ds_data);
 	}
 
-	ds_tmp = NULL;
+	ds_data = NULL;
 
 	sync_check_close();
 	return(ret);
@@ -2240,7 +2123,7 @@ decrypt_decompress()
 	}
 
 	/* copy the rest of tablespaces */
-	ds_ctxt *ds_tmp = ds_create(".", DS_TYPE_LOCAL);
+	ds_data = ds_create(".", DS_TYPE_LOCAL);
 
 	it = datadir_iter_new(".", false);
 
@@ -2253,11 +2136,11 @@ decrypt_decompress()
 		datadir_iter_free(it);
 	}
 
-	if (ds_tmp != NULL) {
-		ds_destroy(ds_tmp);
+	if (ds_data != NULL) {
+		ds_destroy(ds_data);
 	}
 
-	ds_tmp = NULL;
+	ds_data = NULL;
 
 	sync_check_close();
 
@@ -2269,9 +2152,7 @@ decrypt_decompress()
   Do not copy the Innodb files (ibdata1, redo log files),
   as this is done in a separate step.
 */
-static bool backup_files_from_datadir(ds_ctxt_t *ds_data,
-                                      const char *dir_path,
-                                      const char *prefix)
+static bool backup_files_from_datadir(const char *dir_path)
 {
 	os_file_dir_t dir = os_file_opendir(dir_path);
 	if (dir == IF_WIN(INVALID_HANDLE_VALUE, nullptr)) return false;
@@ -2287,7 +2168,8 @@ static bool backup_files_from_datadir(ds_ctxt_t *ds_data,
 		if (!pname)
 			pname = info.name;
 
-		if (!starts_with(pname, prefix))
+		if (!starts_with(pname, "aws-kms-key") &&
+			!starts_with(pname, "aria_log"))
 			/* For ES exchange the above line with the following code:
 			(!xtrabackup_prepare || !xtrabackup_incremental_dir ||
 				!starts_with(pname, "aria_log")))
@@ -2300,7 +2182,7 @@ static bool backup_files_from_datadir(ds_ctxt_t *ds_data,
 
 		std::string full_path(dir_path);
 		full_path.append(1, OS_PATH_SEPARATOR).append(info.name);
-		if (!(ret = ds_data->copy_file(full_path.c_str() , info.name, 1)))
+		if (!(ret = copy_file(ds_data, full_path.c_str() , info.name, 1)))
 			break;
 	}
 	os_file_closedir(dir);
@@ -2350,14 +2232,13 @@ static char *trim_trailing_dir_sep(char *path)
 Create a file hardlink.
 @return true on success, false on error.
 */
-bool
-ds_ctxt_t::make_hardlink(const char *from_path, const char *to_path)
+static bool make_hardlink(const char *from_path, const char *to_path)
 {
 	DBUG_EXECUTE_IF("no_hardlinks", return false;);
 	char to_path_full[FN_REFLEN];
 	if (!is_abs_path(to_path))
 	{
-		fn_format(to_path_full, to_path, root, "", MYF(MY_RELATIVE_PATH));
+		fn_format(to_path_full, to_path, ds_data->root, "", MYF(MY_RELATIVE_PATH));
 	}
 	else
 	{
@@ -2378,9 +2259,7 @@ ds_ctxt_t::make_hardlink(const char *from_path, const char *to_path)
  Has optimization that allows to use hardlinks when possible
  (source and destination are directories on the same device)
 */
-void
-ds_ctxt_t::copy_or_move_dir(const char *from, const char *to,
-                            bool do_copy, bool allow_hardlinks)
+static void copy_or_move_dir(const char *from, const char *to, bool do_copy, bool allow_hardlinks)
 {
 	datadir_node_t node;
 	datadir_node_init(&node);
@@ -2408,8 +2287,8 @@ ds_ctxt_t::copy_or_move_dir(const char *from, const char *to,
 		if (!rc) 
 		{
 			rc = (do_copy ?
-				copy_file(from_path, to_path, 1) :
-				move_file(from_path, node.filepath_rel,
+				copy_file(ds_data, from_path, to_path, 1) :
+				move_file(ds_data, from_path, node.filepath_rel,
 					to, 1));
 		}
 		if (!rc)
@@ -2506,7 +2385,7 @@ static void rocksdb_create_checkpoint()
   remove temp.checkpoint directory (in server's datadir)
   and release user level lock acquired inside rocksdb_create_checkpoint().
 */
-static void rocksdb_backup_checkpoint(ds_ctxt *ds_data)
+static void rocksdb_backup_checkpoint()
 {
 	msg("Backing up rocksdb files.");
 	char rocksdb_backup_dir[FN_REFLEN];
@@ -2518,7 +2397,7 @@ static void rocksdb_backup_checkpoint(ds_ctxt *ds_data)
 			die("Can't create rocksdb backup directory %s", rocksdb_backup_dir);
 		}
 	}
-	ds_data->copy_or_move_dir(rocksdb_checkpoint_dir, ROCKSDB_BACKUP_DIR, true, backup_to_directory);
+	copy_or_move_dir(rocksdb_checkpoint_dir, ROCKSDB_BACKUP_DIR, true, backup_to_directory);
 	rocksdb_remove_checkpoint_directory();
 	rocksdb_unlock_checkpoint();
 }
@@ -2526,7 +2405,7 @@ static void rocksdb_backup_checkpoint(ds_ctxt *ds_data)
 /*
   Copies #rocksdb directory to the $rockdb_data_dir, on copy-back
 */
-static void rocksdb_copy_back(ds_ctxt *ds_data) {
+static void rocksdb_copy_back() {
 	if (access(ROCKSDB_BACKUP_DIR, 0))
 		return;
 	char rocksdb_home_dir[FN_REFLEN];
@@ -2538,5 +2417,5 @@ static void rocksdb_copy_back(ds_ctxt *ds_data) {
 		xb_rocksdb_datadir?trim_dotslash(xb_rocksdb_datadir): ROCKSDB_BACKUP_DIR);
 	}
 	mkdirp(rocksdb_home_dir, 0777, MYF(0));
-	ds_data->copy_or_move_dir(ROCKSDB_BACKUP_DIR, rocksdb_home_dir, xtrabackup_copy_back, xtrabackup_copy_back);
+	copy_or_move_dir(ROCKSDB_BACKUP_DIR, rocksdb_home_dir, xtrabackup_copy_back, xtrabackup_copy_back);
 }
