@@ -15,13 +15,11 @@
 
 #include "wsrep_client_service.h"
 #include "wsrep_high_priority_service.h"
-#include "wsrep_applier.h" /* wsrep_apply_events() */
 #include "wsrep_binlog.h"  /* wsrep_dump_rbr_buf() */
 #include "wsrep_schema.h"  /* remove_fragments() */
 #include "wsrep_thd.h"
 #include "wsrep_xid.h"
 #include "wsrep_trans_observer.h"
-#include "wsrep_server_state.h"
 
 #include "sql_base.h"    /* close_temporary_table() */
 #include "sql_class.h"   /* THD */
@@ -283,18 +281,11 @@ enum wsrep::provider::status Wsrep_client_service::replay()
     original THD state during replication event applying.
    */
   THD *replayer_thd= new THD(true, true);
-  // Replace the security context of the replayer with the security context
-  // of the original THD. Since security context class doesn't have proper
-  // copy constructors, we need to store the original one and set it back
-  // before destruction so that THD desctruction doesn't cause double-free
-  // on the replaced security context.
-  Security_context old_ctx = replayer_thd->main_security_ctx;
-  replayer_thd->main_security_ctx = m_thd->main_security_ctx;
   replayer_thd->thread_stack= m_thd->thread_stack;
   replayer_thd->real_id= pthread_self();
   replayer_thd->prior_thr_create_utime=
       replayer_thd->start_utime= microsecond_interval_timer();
-  replayer_thd->mark_connection_idle();
+  replayer_thd->set_command(COM_SLEEP);
   replayer_thd->reset_for_next_command(true);
 
   enum wsrep::provider::status ret;
@@ -306,7 +297,6 @@ enum wsrep::provider::status Wsrep_client_service::replay()
     replayer_service.replay_status(ret);
   }
 
-  replayer_thd->main_security_ctx = old_ctx;
   delete replayer_thd;
   DBUG_RETURN(ret);
 }
@@ -362,7 +352,8 @@ int Wsrep_client_service::bf_rollback()
   int ret= (trans_rollback_stmt(m_thd) || trans_rollback(m_thd));
   if (m_thd->locked_tables_mode && m_thd->lock)
   {
-    m_thd->locked_tables_list.unlock_locked_tables(m_thd);
+    if (m_thd->locked_tables_list.unlock_locked_tables(m_thd))
+      ret= 1;
     m_thd->variables.option_bits&= ~OPTION_TABLE_LOCK;
   }
   if (m_thd->global_read_lock.is_acquired())
@@ -370,6 +361,8 @@ int Wsrep_client_service::bf_rollback()
     m_thd->global_read_lock.unlock_global_read_lock(m_thd);
   }
   m_thd->release_transactional_locks();
+  mysql_ull_cleanup(m_thd);
+  m_thd->mdl_context.release_explicit_locks();
 
   DBUG_RETURN(ret);
 }
